#pragma once
#include <list>
#include <string>
#include "collisionPoint.h"
#include "CriticalParts.h"

class scheduler
{
public:
    scheduler();
    ~scheduler();

    //Read the input file (kisautok.txt) and store the read information in the "cars" list
    void readInput();

    void simulateOneStep(int currentTime);

    void writeResults();

    void sumEnergy();

    void writeEnergyResult();

private:
    std::list<car> cars;
    std::list<collisionPoint> collisionPoints;
    std::list<criticalPart> criticalParts;

    bool isWorking;

    int energy;

    //Get the car which has the same color as parameter, or a car with "WrongColor" if there is no car with same color
    car& getCarByColor(CarColor color);

    //Get the Car Color based on the input string (piros, sarga, zold, fekete, kek)
    CarColor mapColor(std::string color);

    //Get the color string based on the input Car Color
    std::string colorString(CarColor color);

    //Get the Direction based on the input string (+, -)
    Direction mapDirection(std::string direction);

    //Simulate one step by cars
    void carStep(int currentTime);

    //Check if the added car arrives at collisionpoint or not in a defined position
    //pos1 - the position in car1 coordinate system
    void checkCollisionPoint(car car1, int pos1, int time, collisionPoint& cp);

    //Check if the added two cars have a collision or not in a defined position
    //pos1 - the position in car1 coordinate system
    //pos2 - the position in car2 coordinate system
    bool checkCollisionPoint(car car1, int pos1, car car2, int pos2);

    //If any car arrive at collision point, the arrive list shall be updated
    void updateArrives(int time);

    //Chech if there is a collision
    void checkCollision();

    //Refresh the waiting parameters
    void refreshWaitingParameters();

    //Check if a position is between two other defined position
    bool isPosBetweenTwoPos(int check_pos, int pos1, int pos2);

    /*
    Critical Part: O-----O
                  CS1   CS2

    Check the critical situations, if car shall wait

    Input parameter:
     - Car1, Car2: the cars shall be checked
     - car1_pos1: CS1 position in car1 coordinate system
     - car2_pos1: CS1 position in car2 coordinate system
     - car2_pos2: CS2 position in car2 coordinate system
     - car1_criticalDir1: critical direction in CS1 for car1
     - car2_criticalDir2: critical direction in CS2 for car2
    */
    void checkOnCarInOneCriticalPosition(car& Car1, car& Car2, int car1_pos1, int car2_pos1, int car2_pos2, Direction car1_criticalDir1, Direction car2_criticalDir2);
};

